@@ -18,8 +18,9 @@ from __future__ import annotations
 
 import datetime
 import uuid
+from collections.abc import Iterable
 from copy import copy
-from typing import Annotated, Any, Dict, List, Literal, Optional, Union
+from typing import Annotated, Any, Literal
 
 from pydantic import Field, field_serializer, field_validator, model_validator
 from pydantic import ValidationError as PydanticValidationError
@@ -68,7 +69,7 @@ DEFAULT_SCHEMA_ID = 0
 SUPPORTED_TABLE_FORMAT_VERSION = 2
 
 
-def cleanup_snapshot_id(data: Dict[str, Any]) -> Dict[str, Any]:
+def cleanup_snapshot_id(data: dict[str, Any]) -> dict[str, Any]:
     """Run before validation."""
     if CURRENT_SNAPSHOT_ID in data and data[CURRENT_SNAPSHOT_ID] == -1:
         # We treat -1 and None the same, by cleaning this up
@@ -92,7 +93,7 @@ def check_partition_specs(table_metadata: TableMetadata) -> TableMetadata:
     """Check if the default-spec-id is present in partition-specs."""
     default_spec_id = table_metadata.default_spec_id
 
-    partition_specs: List[PartitionSpec] = table_metadata.partition_specs
+    partition_specs: list[PartitionSpec] = table_metadata.partition_specs
     for spec in partition_specs:
         if spec.spec_id == default_spec_id:
             return table_metadata
@@ -105,7 +106,7 @@ def check_sort_orders(table_metadata: TableMetadata) -> TableMetadata:
     default_sort_order_id: int = table_metadata.default_sort_order_id
 
     if default_sort_order_id != UNSORTED_SORT_ORDER_ID:
-        sort_orders: List[SortOrder] = table_metadata.sort_orders
+        sort_orders: list[SortOrder] = table_metadata.sort_orders
         for sort_order in sort_orders:
             if sort_order.order_id == default_sort_order_id:
                 return table_metadata
@@ -151,39 +152,39 @@ class TableMetadataCommonFields(IcebergBaseModel):
     This is used to ensure fields are always assigned an unused ID
     when evolving schemas."""
 
-    schemas: List[Schema] = Field(default_factory=list)
+    schemas: list[Schema] = Field(default_factory=list)
     """A list of schemas, stored as objects with schema-id."""
 
     current_schema_id: int = Field(alias="current-schema-id", default=DEFAULT_SCHEMA_ID)
     """ID of the table’s current schema."""
 
-    partition_specs: List[PartitionSpec] = Field(alias="partition-specs", default_factory=list)
+    partition_specs: list[PartitionSpec] = Field(alias="partition-specs", default_factory=list)
     """A list of partition specs, stored as full partition spec objects."""
 
     default_spec_id: int = Field(alias="default-spec-id", default=INITIAL_SPEC_ID)
     """ID of the “current” spec that writers should use by default."""
 
-    last_partition_id: Optional[int] = Field(alias="last-partition-id", default=None)
+    last_partition_id: int | None = Field(alias="last-partition-id", default=None)
     """An integer; the highest assigned partition field ID across all
     partition specs for the table. This is used to ensure partition fields
     are always assigned an unused ID when evolving specs."""
 
-    properties: Dict[str, str] = Field(default_factory=dict)
+    properties: dict[str, str] = Field(default_factory=dict)
     """A string to string map of table properties. This is used to
     control settings that affect reading and writing and is not intended
     to be used for arbitrary metadata. For example, commit.retry.num-retries
     is used to control the number of commit retries."""
 
-    current_snapshot_id: Optional[int] = Field(alias="current-snapshot-id", default=None)
+    current_snapshot_id: int | None = Field(alias="current-snapshot-id", default=None)
     """ID of the current table snapshot."""
 
-    snapshots: List[Snapshot] = Field(default_factory=list)
+    snapshots: list[Snapshot] = Field(default_factory=list)
     """A list of valid snapshots. Valid snapshots are snapshots for which
     all data files exist in the file system. A data file must not be
     deleted from the file system until the last snapshot in which it was
     listed is garbage collected."""
 
-    snapshot_log: List[SnapshotLogEntry] = Field(alias="snapshot-log", default_factory=list)
+    snapshot_log: list[SnapshotLogEntry] = Field(alias="snapshot-log", default_factory=list)
     """A list (optional) of timestamp and snapshot ID pairs that encodes
     changes to the current snapshot for the table. Each time the
     current-snapshot-id is changed, a new entry should be added with the
@@ -191,7 +192,7 @@ class TableMetadataCommonFields(IcebergBaseModel):
     expired from the list of valid snapshots, all entries before a snapshot
     that has expired should be removed."""
 
-    metadata_log: List[MetadataLogEntry] = Field(alias="metadata-log", default_factory=list)
+    metadata_log: list[MetadataLogEntry] = Field(alias="metadata-log", default_factory=list)
     """A list (optional) of timestamp and metadata file location pairs that
     encodes changes to the previous metadata files for the table. Each time
     a new metadata file is created, a new entry of the previous metadata
@@ -199,7 +200,7 @@ class TableMetadataCommonFields(IcebergBaseModel):
     remove oldest metadata log entries and keep a fixed-size log of the most
     recent entries after a commit."""
 
-    sort_orders: List[SortOrder] = Field(alias="sort-orders", default_factory=list)
+    sort_orders: list[SortOrder] = Field(alias="sort-orders", default_factory=list)
     """A list of sort orders, stored as full sort order objects."""
 
     default_sort_order_id: int = Field(alias="default-sort-order-id", default=UNSORTED_SORT_ORDER_ID)
@@ -207,14 +208,14 @@ class TableMetadataCommonFields(IcebergBaseModel):
     writers, but is not used when reading because reads use the specs stored
      in manifest files."""
 
-    refs: Dict[str, SnapshotRef] = Field(default_factory=dict)
+    refs: dict[str, SnapshotRef] = Field(default_factory=dict)
     """A map of snapshot references.
     The map keys are the unique snapshot reference names in the table,
     and the map values are snapshot reference objects.
     There is always a main branch reference pointing to the
     current-snapshot-id even if the refs map is null."""
 
-    statistics: List[StatisticsFile] = Field(default_factory=list)
+    statistics: list[StatisticsFile] = Field(default_factory=list)
     """A optional list of table statistics files.
     Table statistics files are valid Puffin files. Statistics are
     informational. A reader can choose to ignore statistics
@@ -222,7 +223,7 @@ class TableMetadataCommonFields(IcebergBaseModel):
     table correctly. A table can contain many statistics files
     associated with different table snapshots."""
 
-    partition_statistics: List[PartitionStatisticsFile] = Field(alias="partition-statistics", default_factory=list)
+    partition_statistics: list[PartitionStatisticsFile] = Field(alias="partition-statistics", default_factory=list)
     """A optional list of partition statistics files.
     Partition statistics are not required for reading or planning
     and readers may ignore them. Each table snapshot may be associated
@@ -232,14 +233,14 @@ class TableMetadataCommonFields(IcebergBaseModel):
 
     # validators
     @field_validator("properties", mode="before")
-    def transform_properties_dict_value_to_str(cls, properties: Properties) -> Dict[str, str]:
+    def transform_properties_dict_value_to_str(cls, properties: Properties) -> dict[str, str]:
         return transform_dict_value_to_str(properties)
 
-    def snapshot_by_id(self, snapshot_id: int) -> Optional[Snapshot]:
+    def snapshot_by_id(self, snapshot_id: int) -> Snapshot | None:
         """Get the snapshot by snapshot_id."""
         return next((snapshot for snapshot in self.snapshots if snapshot.snapshot_id == snapshot_id), None)
 
-    def schema_by_id(self, schema_id: int) -> Optional[Schema]:
+    def schema_by_id(self, schema_id: int) -> Schema | None:
         """Get the schema by schema_id."""
         return next((schema for schema in self.schemas if schema.schema_id == schema_id), None)
 
@@ -247,7 +248,7 @@ class TableMetadataCommonFields(IcebergBaseModel):
         """Return the schema for this table."""
         return next(schema for schema in self.schemas if schema.schema_id == self.current_schema_id)
 
-    def name_mapping(self) -> Optional[NameMapping]:
+    def name_mapping(self) -> NameMapping | None:
         """Return the table's field-id NameMapping."""
         if name_mapping_json := self.properties.get("schema.name-mapping.default"):
             return parse_mapping_from_json(name_mapping_json)
@@ -258,22 +259,27 @@ class TableMetadataCommonFields(IcebergBaseModel):
         """Return the partition spec of this table."""
         return next(spec for spec in self.partition_specs if spec.spec_id == self.default_spec_id)
 
-    def specs(self) -> Dict[int, PartitionSpec]:
+    def specs(self) -> dict[int, PartitionSpec]:
         """Return a dict the partition specs this table."""
         return {spec.spec_id: spec for spec in self.partition_specs}
 
-    def specs_struct(self) -> StructType:
-        """Produce a struct of all the combined PartitionSpecs.
+    def specs_struct(self, spec_ids: Iterable[int] | None = None) -> StructType:
+        """Produce a struct of the combined PartitionSpecs.
 
         The partition fields should be optional: Partition fields may be added later,
         in which case not all files would have the result field, and it may be null.
 
-        :return: A StructType that represents all the combined PartitionSpecs of the table
+        Args:
+            spec_ids: Optional iterable of spec IDs to include. When not provided,
+                all table specs are used.
+
+        :return: A StructType that represents the combined PartitionSpecs of the table
         """
         specs = self.specs()
+        selected_specs = specs.values() if spec_ids is None else [specs[spec_id] for spec_id in spec_ids if spec_id in specs]
 
         # Collect all the fields
-        struct_fields = {field.field_id: field for spec in specs.values() for field in spec.fields}
+        struct_fields = {field.field_id: field for spec in selected_specs for field in spec.fields}
 
         schema = self.schema()
 
@@ -295,7 +301,7 @@ class TableMetadataCommonFields(IcebergBaseModel):
 
         return snapshot_id
 
-    def snapshot_by_name(self, name: Optional[str]) -> Optional[Snapshot]:
+    def snapshot_by_name(self, name: str | None) -> Snapshot | None:
         """Return the snapshot referenced by the given name or null if no such reference exists."""
         if name is None:
             name = MAIN_BRANCH
@@ -303,7 +309,7 @@ class TableMetadataCommonFields(IcebergBaseModel):
             return self.snapshot_by_id(ref.snapshot_id)
         return None
 
-    def current_snapshot(self) -> Optional[Snapshot]:
+    def current_snapshot(self) -> Snapshot | None:
         """Get the current snapshot for this table, or None if there is no current snapshot."""
         if self.current_snapshot_id is not None:
             return self.snapshot_by_id(self.current_snapshot_id)
@@ -312,18 +318,22 @@ class TableMetadataCommonFields(IcebergBaseModel):
     def next_sequence_number(self) -> int:
         return self.last_sequence_number + 1 if self.format_version > 1 else INITIAL_SEQUENCE_NUMBER
 
-    def sort_order_by_id(self, sort_order_id: int) -> Optional[SortOrder]:
+    def sort_order(self) -> SortOrder:
+        """Get the current sort order for this table, or UNSORTED_SORT_ORDER if there is no sort order."""
+        return self.sort_order_by_id(self.default_sort_order_id) or UNSORTED_SORT_ORDER
+
+    def sort_order_by_id(self, sort_order_id: int) -> SortOrder | None:
         """Get the sort order by sort_order_id."""
         return next((sort_order for sort_order in self.sort_orders if sort_order.order_id == sort_order_id), None)
 
     @field_serializer("current_snapshot_id")
-    def serialize_current_snapshot_id(self, current_snapshot_id: Optional[int]) -> Optional[int]:
+    def serialize_current_snapshot_id(self, current_snapshot_id: int | None) -> int | None:
         if current_snapshot_id is None and Config().get_bool("legacy-current-snapshot-id"):
             return -1
         return current_snapshot_id
 
     @field_serializer("snapshots")
-    def serialize_snapshots(self, snapshots: List[Snapshot]) -> List[Snapshot]:
+    def serialize_snapshots(self, snapshots: list[Snapshot]) -> list[Snapshot]:
         # Snapshot field `sequence-number` should not be written for v1 metadata
         if self.format_version == 1:
             return [snapshot.model_copy(update={"sequence_number": None}) for snapshot in snapshots]
@@ -337,7 +347,9 @@ def _generate_snapshot_id() -> int:
     """
     rnd_uuid = uuid.uuid4()
     snapshot_id = int.from_bytes(
-        bytes(lhs ^ rhs for lhs, rhs in zip(rnd_uuid.bytes[0:8], rnd_uuid.bytes[8:16])), byteorder="little", signed=True
+        bytes(lhs ^ rhs for lhs, rhs in zip(rnd_uuid.bytes[0:8], rnd_uuid.bytes[8:16], strict=True)),
+        byteorder="little",
+        signed=True,
     )
     snapshot_id = snapshot_id if snapshot_id >= 0 else snapshot_id * -1
 
@@ -359,15 +371,15 @@ class TableMetadataV1(TableMetadataCommonFields, IcebergBaseModel):
     # to the owner of the table.
 
     @model_validator(mode="before")
-    def cleanup_snapshot_id(cls, data: Dict[str, Any]) -> Dict[str, Any]:
+    def cleanup_snapshot_id(cls, data: dict[str, Any]) -> dict[str, Any]:
         return cleanup_snapshot_id(data)
 
     @model_validator(mode="after")
-    def construct_refs(cls, data: TableMetadataV1) -> TableMetadataV1:
-        return construct_refs(data)
+    def construct_refs(self) -> TableMetadataV1:
+        return construct_refs(self)
 
     @model_validator(mode="before")
-    def set_v2_compatible_defaults(cls, data: Dict[str, Any]) -> Dict[str, Any]:
+    def set_v2_compatible_defaults(cls, data: dict[str, Any]) -> dict[str, Any]:
         """Set default values to be compatible with the format v2.
 
         Args:
@@ -385,7 +397,7 @@ class TableMetadataV1(TableMetadataCommonFields, IcebergBaseModel):
         return data
 
     @model_validator(mode="before")
-    def construct_schemas(cls, data: Dict[str, Any]) -> Dict[str, Any]:
+    def construct_schemas(cls, data: dict[str, Any]) -> dict[str, Any]:
         """Convert the schema into schemas.
 
         For V1 schemas is optional, and if they aren't set, we'll set them
@@ -404,7 +416,7 @@ class TableMetadataV1(TableMetadataCommonFields, IcebergBaseModel):
         return data
 
     @model_validator(mode="before")
-    def construct_partition_specs(cls, data: Dict[str, Any]) -> Dict[str, Any]:
+    def construct_partition_specs(cls, data: dict[str, Any]) -> dict[str, Any]:
         """Convert the partition_spec into partition_specs.
 
         For V1 partition_specs is optional, and if they aren't set, we'll set them
@@ -439,7 +451,7 @@ class TableMetadataV1(TableMetadataCommonFields, IcebergBaseModel):
         return data
 
     @model_validator(mode="before")
-    def set_sort_orders(cls, data: Dict[str, Any]) -> Dict[str, Any]:
+    def set_sort_orders(cls, data: dict[str, Any]) -> dict[str, Any]:
         """Set the sort_orders if not provided.
 
         For V1 sort_orders is optional, and if they aren't set, we'll set them
@@ -468,7 +480,7 @@ class TableMetadataV1(TableMetadataCommonFields, IcebergBaseModel):
     """The table’s current schema. (Deprecated: use schemas and
     current-schema-id instead)."""
 
-    partition_spec: List[Dict[str, Any]] = Field(alias="partition-spec", default_factory=list)
+    partition_spec: list[dict[str, Any]] = Field(alias="partition-spec", default_factory=list)
     """The table’s current partition spec, stored as only fields.
     Note that this is used by writers to partition data, but is
     not used when reading because reads use the specs stored in
@@ -488,24 +500,24 @@ class TableMetadataV2(TableMetadataCommonFields, IcebergBaseModel):
     """
 
     @model_validator(mode="before")
-    def cleanup_snapshot_id(cls, data: Dict[str, Any]) -> Dict[str, Any]:
+    def cleanup_snapshot_id(cls, data: dict[str, Any]) -> dict[str, Any]:
         return cleanup_snapshot_id(data)
 
     @model_validator(mode="after")
-    def check_schemas(cls, table_metadata: TableMetadata) -> TableMetadata:
-        return check_schemas(table_metadata)
+    def check_schemas(self) -> TableMetadata:
+        return check_schemas(self)
 
     @model_validator(mode="after")
-    def check_partition_specs(cls, table_metadata: TableMetadata) -> TableMetadata:
-        return check_partition_specs(table_metadata)
+    def check_partition_specs(self) -> TableMetadata:
+        return check_partition_specs(self)
 
     @model_validator(mode="after")
-    def check_sort_orders(cls, table_metadata: TableMetadata) -> TableMetadata:
-        return check_sort_orders(table_metadata)
+    def check_sort_orders(self) -> TableMetadata:
+        return check_sort_orders(self)
 
     @model_validator(mode="after")
-    def construct_refs(cls, table_metadata: TableMetadata) -> TableMetadata:
-        return construct_refs(table_metadata)
+    def construct_refs(self) -> TableMetadata:
+        return construct_refs(self)
 
     format_version: Literal[2] = Field(alias="format-version", default=2)
     """An integer version number for the format. Implementations must throw
@@ -532,24 +544,24 @@ class TableMetadataV3(TableMetadataCommonFields, IcebergBaseModel):
     """
 
     @model_validator(mode="before")
-    def cleanup_snapshot_id(cls, data: Dict[str, Any]) -> Dict[str, Any]:
+    def cleanup_snapshot_id(cls, data: dict[str, Any]) -> dict[str, Any]:
         return cleanup_snapshot_id(data)
 
     @model_validator(mode="after")
-    def check_schemas(cls, table_metadata: TableMetadata) -> TableMetadata:
-        return check_schemas(table_metadata)
+    def check_schemas(self) -> TableMetadata:
+        return check_schemas(self)
 
     @model_validator(mode="after")
-    def check_partition_specs(cls, table_metadata: TableMetadata) -> TableMetadata:
-        return check_partition_specs(table_metadata)
+    def check_partition_specs(self) -> TableMetadata:
+        return check_partition_specs(self)
 
     @model_validator(mode="after")
-    def check_sort_orders(cls, table_metadata: TableMetadata) -> TableMetadata:
-        return check_sort_orders(table_metadata)
+    def check_sort_orders(self) -> TableMetadata:
+        return check_sort_orders(self)
 
     @model_validator(mode="after")
-    def construct_refs(cls, table_metadata: TableMetadata) -> TableMetadata:
-        return construct_refs(table_metadata)
+    def construct_refs(self) -> TableMetadata:
+        return construct_refs(self)
 
     format_version: Literal[3] = Field(alias="format-version", default=3)
     """An integer version number for the format. Implementations must throw
@@ -559,16 +571,14 @@ class TableMetadataV3(TableMetadataCommonFields, IcebergBaseModel):
     """The table’s highest assigned sequence number, a monotonically
     increasing long that tracks the order of snapshots in a table."""
 
-    next_row_id: Optional[int] = Field(alias="next-row-id", default=None)
+    next_row_id: int | None = Field(alias="next-row-id", default=None)
     """A long higher than all assigned row IDs; the next snapshot's `first-row-id`."""
 
-    def model_dump_json(
-        self, exclude_none: bool = True, exclude: Optional[Any] = None, by_alias: bool = True, **kwargs: Any
-    ) -> str:
+    def model_dump_json(self, exclude_none: bool = True, exclude: Any | None = None, by_alias: bool = True, **kwargs: Any) -> str:
         raise NotImplementedError("Writing V3 is not yet supported, see: https://github.com/apache/iceberg-python/issues/1551")
 
 
-TableMetadata = Annotated[Union[TableMetadataV1, TableMetadataV2, TableMetadataV3], Field(discriminator="format_version")]
+TableMetadata = Annotated[TableMetadataV1 | TableMetadataV2 | TableMetadataV3, Field(discriminator="format_version")]
 
 
 def new_table_metadata(
@@ -577,7 +587,7 @@ def new_table_metadata(
     sort_order: SortOrder,
     location: str,
     properties: Properties = EMPTY_DICT,
-    table_uuid: Optional[uuid.UUID] = None,
+    table_uuid: uuid.UUID | None = None,
 ) -> TableMetadata:
     from pyiceberg.table import TableProperties
 
@@ -655,7 +665,7 @@ class TableMetadataUtil:
             raise ValidationError(e) from e
 
     @staticmethod
-    def parse_obj(data: Dict[str, Any]) -> TableMetadata:
+    def parse_obj(data: dict[str, Any]) -> TableMetadata:
         if "format-version" not in data:
             raise ValidationError(f"Missing format-version in TableMetadata: {data}")
         format_version = data["format-version"]
@@ -673,7 +683,8 @@ class TableMetadataUtil:
     def _construct_without_validation(table_metadata: TableMetadata) -> TableMetadata:
         """Construct table metadata from an existing table without performing validation.
 
-        This method is useful during a sequence of table updates when the model needs to be re-constructed but is not yet ready for validation.
+        This method is useful during a sequence of table updates when the model needs to be
+        re-constructed but is not yet ready for validation.
         """
         if table_metadata.format_version is None:
             raise ValidationError(f"Missing format-version in TableMetadata: {table_metadata}")
