@@ -17,8 +17,6 @@
 
 from typing import (
     Any,
-    Dict,
-    Optional,
 )
 
 from pyiceberg.typedef import Properties
@@ -28,10 +26,10 @@ HEADER_PREFIX = "header."
 
 
 def property_as_int(
-    properties: Dict[str, str],
+    properties: dict[str, str],
     property_name: str,
-    default: Optional[int] = None,
-) -> Optional[int]:
+    default: int | None = None,
+) -> int | None:
     if value := properties.get(property_name):
         try:
             return int(value)
@@ -42,10 +40,10 @@ def property_as_int(
 
 
 def property_as_float(
-    properties: Dict[str, str],
+    properties: dict[str, str],
     property_name: str,
-    default: Optional[float] = None,
-) -> Optional[float]:
+    default: float | None = None,
+) -> float | None:
     if value := properties.get(property_name):
         try:
             return float(value)
@@ -56,7 +54,7 @@ def property_as_float(
 
 
 def property_as_bool(
-    properties: Dict[str, str],
+    properties: dict[str, str],
     property_name: str,
     default: bool,
 ) -> bool:
@@ -71,7 +69,7 @@ def property_as_bool(
 def get_first_property_value(
     properties: Properties,
     *property_names: str,
-) -> Optional[Any]:
+) -> Any | None:
     for property_name in property_names:
         if property_value := properties.get(property_name):
             return property_value
